@@ -510,6 +510,7 @@ final class DocumentsWriter {
         }
 
         deletesInRAM.clear();
+        deletesFlushed.clear();
 
         openFiles.clear();
 
@@ -537,6 +538,9 @@ final class DocumentsWriter {
     } finally {
       aborting = false;
       notifyAll();
+      if (infoStream != null) {
+        message("docWriter: done abort; abortedFiles=" + abortedFiles);
+      }
     }
   }
 
@@ -660,6 +664,10 @@ final class DocumentsWriter {
     return flushState.numDocs;
   }
 
+  Collection getFlushedFiles() {
+    return flushState.flushedFiles;
+  }
+
   /** Build compound file for the segment we just flushed */
   void createCompoundFile(String segment) throws IOException {
     
