@@ -75,7 +75,7 @@ import java.util.Map;
  <code>IndexReader</code> instance; use your own
  (non-Lucene) objects instead.
  
- @version $Id: IndexReader.java 826049 2009-10-16 19:28:55Z mikemccand $
+ @version $Id: IndexReader.java 950612 2010-06-02 16:03:19Z mikemccand $
 */
 public abstract class IndexReader implements Cloneable {
 
@@ -1495,6 +1495,12 @@ public abstract class IndexReader implements Cloneable {
     return this;
   }
 
+  /** Expert.  Warning: this returns null if the reader has
+   *  no deletions */
+  public Object getDeletesCacheKey() {
+    return this;
+  }
+
   /** Returns the number of unique terms (across all fields)
    *  in this reader.
    *
