@@ -570,7 +570,7 @@ public final class SegmentInfos extends Vector {
 
           // Method 2: open segments.gen and read its
           // contents.  Then we take the larger of the two
-          // gen's.  This way, if either approach is hitting
+          // gens.  This way, if either approach is hitting
           // a stale cache (NFS) we have a better chance of
           // getting the right generation.
           long genB = -1;
@@ -681,9 +681,7 @@ public final class SegmentInfos extends Vector {
 
         try {
           Object v = doBody(segmentFileName);
-          if (exc != null) {
-            message("success on " + segmentFileName);
-          }
+          message("success on " + segmentFileName);
           return v;
         } catch (IOException err) {
 
