@@ -74,14 +74,16 @@ final class TermVectorsTermsWriterPerField extends TermsHashConsumerPerField {
         assert perThread.doc.numVectorFields == 0;
         assert 0 == perThread.doc.perDocTvf.length();
         assert 0 == perThread.doc.perDocTvf.getFilePointer();
-      } else {
-        assert perThread.doc.docID == docState.docID;
+      }
 
-        if (termsHashPerField.numPostings != 0)
-          // Only necessary if previous doc hit a
-          // non-aborting exception while writing vectors in
-          // this field:
-          termsHashPerField.reset();
+      assert perThread.doc.docID == docState.docID;
+
+      if (termsHashPerField.numPostings != 0) {
+        // Only necessary if previous doc hit a
+        // non-aborting exception while writing vectors in
+        // this field:
+        termsHashPerField.reset();
+        perThread.termsHashPerThread.reset(false);
       }
     }
 
@@ -96,7 +98,7 @@ final class TermVectorsTermsWriterPerField extends TermsHashConsumerPerField {
   /** Called once per field per document if term vectors
    *  are enabled, to write the vectors to
    *  RAMOutputStream, which is then quickly flushed to
-   *  * the real term vectors files in the Directory. */
+   *  the real term vectors files in the Directory. */
   void finish() throws IOException {
 
     assert docState.testPoint("TermVectorsTermsWriterPerField.finish start");
@@ -186,6 +188,12 @@ final class TermVectorsTermsWriterPerField extends TermsHashConsumerPerField {
     }
 
     termsHashPerField.reset();
+
+    // NOTE: we clear, per-field, at the thread level,
+    // because term vectors fully write themselves on each
+    // field; this saves RAM (eg if large doc has two large
+    // fields w/ term vectors on) because we recycle/reuse
+    // all RAM after each field:
     perThread.termsHashPerThread.reset(false);
   }
 
