@@ -35,7 +35,7 @@ import java.text.DecimalFormat;
  * <p>Created: May 19, 2004 11:13:14 AM
  *
  * @since   lucene 1.4
- * @version $Id: FieldCache.java 811070 2009-09-03 18:31:41Z hossman $
+ * @version $Id: FieldCache.java 950451 2010-06-02 09:33:57Z mikemccand $
  * @see org.apache.lucene.util.FieldCacheSanityChecker
  */
 public interface FieldCache {
@@ -616,6 +616,15 @@ public interface FieldCache {
    */
   public abstract void purgeAllCaches();
 
+  /**
+   * Expert: drops all cache entries associated with this
+   * reader.  NOTE: this reader must precisely match the
+   * reader that the cache entry is keyed on. If you pass a
+   * top-level reader, it usually will have no effect as
+   * Lucene now caches at the segment reader level.
+   */
+  public abstract void purge(IndexReader r);
+
   /**
    * If non-null, FieldCacheImpl will warn whenever
    * entries are created that are not sane according to
