@@ -1012,7 +1012,7 @@ public class TestIndexReader extends LuceneTestCase
           String[] startFiles = dir.listAll();
           SegmentInfos infos = new SegmentInfos();
           infos.read(dir);
-          new IndexFileDeleter(dir, new KeepOnlyLastCommitDeletionPolicy(), infos, null, null);
+          new IndexFileDeleter(dir, new KeepOnlyLastCommitDeletionPolicy(), infos, null, null, null);
           String[] endFiles = dir.listAll();
 
           Arrays.sort(startFiles);
