@@ -164,7 +164,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     assertTrue(r1.isCurrent());
 
     writer.commit();
-    assertTrue(r1.isCurrent());
+    assertFalse(r1.isCurrent());
 
     assertEquals(200, r1.maxDoc());
 
@@ -699,7 +699,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
 
   // Stress test reopen during addIndexes
   public void testDuringAddIndexes() throws Exception {
-    Directory dir1 = new MockRAMDirectory();
+    MockRAMDirectory dir1 = new MockRAMDirectory();
     final IndexWriter writer = new IndexWriter(dir1, new WhitespaceAnalyzer(),
                                                IndexWriter.MaxFieldLength.LIMITED);
     writer.setInfoStream(infoStream);
@@ -758,10 +758,12 @@ public class TestIndexWriterReader extends LuceneTestCase {
     }
 
     assertEquals(0, excs.size());
+    r.close();
+    assertEquals(0, dir1.getOpenDeletedFiles().size());
+
     writer.close();
 
     _TestUtil.checkIndex(dir1);
-    r.close();
     dir1.close();
   }
 
@@ -793,12 +795,12 @@ public class TestIndexWriterReader extends LuceneTestCase {
             final Random r = new Random();
             while(System.currentTimeMillis() < endTime) {
               try {
-                for(int i=0;i<10;i++) {
-                  writer.addDocument(createDocument(10*count+i, "test", 4));
+                for(int docUpto=0;docUpto<10;docUpto++) {
+                  writer.addDocument(createDocument(10*count+docUpto, "test", 4));
                 }
                 count++;
                 final int limit = count*10;
-                for(int i=0;i<5;i++) {
+                for(int delUpto=0;delUpto<5;delUpto++) {
                   int x = r.nextInt(limit);
                   writer.deleteDocuments(new Term("field3", "b"+x));
                 }
