@@ -26,9 +26,19 @@ import org.apache.lucene.index.CheckIndex;
 import org.apache.lucene.store.Directory;
 import java.io.ByteArrayOutputStream;
 import java.io.PrintStream;
+import java.util.Random;
 
 public class _TestUtil {
 
+  /** Returns temp dir, containing String arg in its name;
+   *  does not create the directory. */
+  public static File getTempDir(String desc) {
+    String tempDir = System.getProperty("java.io.tmpdir");
+    if (tempDir == null)
+      throw new RuntimeException("java.io.tmpdir undefined, cannot run test");
+    return new File(tempDir, desc + "." + new Random().nextLong());
+  }
+
   public static void rmDir(File dir) throws IOException {
     if (dir.exists()) {
       File[] files = dir.listFiles();
@@ -71,4 +81,40 @@ public class _TestUtil {
       return true;
   }
 
+  /** Use only for testing.
+   *  @deprecated -- in 3.0 we can use Arrays.toString
+   *  instead */
+  public static String arrayToString(int[] array) {
+    StringBuffer buf = new StringBuffer();
+    buf.append("[");
+    for(int i=0;i<array.length;i++) {
+      if (i > 0) {
+        buf.append(" ");
+      }
+      buf.append(array[i]);
+    }
+    buf.append("]");
+    return buf.toString();
+  }
+
+  /** Use only for testing.
+   *  @deprecated -- in 3.0 we can use Arrays.toString
+   *  instead */
+  public static String arrayToString(Object[] array) {
+    StringBuffer buf = new StringBuffer();
+    buf.append("[");
+    for(int i=0;i<array.length;i++) {
+      if (i > 0) {
+        buf.append(" ");
+      }
+      buf.append(array[i]);
+    }
+    buf.append("]");
+    return buf.toString();
+  }
+
+  public static int getRandomSocketPort() {
+    return 1024 + new Random().nextInt(64512);
+  }
+
 }
